@@ -132,17 +132,17 @@ sub set_logger {
 
 	if(ref($_[0]) eq 'HASH') {
 		%args = %{$_[0]};
-	} elsif(!ref($_[0])) {
-		Carp::croak('Usage: set_logger(logger => $logger)');
 	} elsif(scalar(@_) % 2 == 0) {
 		%args = @_;
-	} else {
+	} elsif((scalar(@_) == 1) && !ref($_[0])) {
 		$args{'logger'} = shift;
 	}
 
-	$self->{'logger'} = $args{'logger'};
-
-	return $self;
+	if(defined($args{'logger'})) {
+		$self->{'logger'} = $args{'logger'};
+		return $self;
+	}
+	Carp::croak('Usage: execute(query => $query)')
 }
 
 # Open the database.
@@ -542,11 +542,9 @@ sub execute {
 
 	if(ref($_[0]) eq 'HASH') {
 		%args = %{$_[0]};
-	} elsif(ref($_[0])) {
-		Carp::croak('Usage: execute(query => $query)');
 	} elsif(scalar(@_) % 2 == 0) {
 		%args = @_;
-	} else {
+	} elsif((scalar(@_) == 1) && !ref($_[0])) {
 		$args{'query'} = shift;
 	}
 
